Base64 (py4j-java 0.10.9.7 API)

Package py4j

## Class Base64

- java.lang.Object
- - py4j.Base64

-     public class Base64
      extends java.lang.Object

  A very fast and memory efficient class to encode and decode to and from BASE64 in full accordance with RFC 2045.
  On Windows XP sp1 with 1.4.2_04 and later ;), this encoder and decoder is about 10 times faster on small arrays (10 - 1000 bytes) and 2-3 times as fast on larger arrays (10000 - 1000000 bytes) compared to `sun.misc.Encoder()/Decoder()`.
  On byte arrays the encoder is about 20% faster than Jakarta Commons Base64 Codec for encode and about 50% faster for decoding large arrays. This implementation is about twice as fast on very small arrays (< 30 bytes). If source/destination is a `String` this version is about three times as fast due to the fact that the Commons Codec result has to be recoded to a `String` from `byte[]`, which is very expensive.
  This encode/decode algorithm doesn't create any temporary arrays as many other codecs do, it only allocates the resulting array. This produces less garbage and it is possible to handle arrays twice as large as algorithms that create a temporary array. (E.g. Jakarta Commons Codec). It is unknown whether Sun's `sun.misc.Encoder()/Decoder()` produce temporary arrays but since performance is quite low it probably does.
  The encoder produces the same output as the Sun one except that the Sun's encoder appends a trailing line separator if the last character isn't a pad. Unclear why but it only adds to the length and is probably a side effect. Both are in conformance with RFC 2045 though.
  Commons codec seem to always att a trailing line separator.
  Note! The encode/decode method pairs (types) come in three versions with the exact same algorithm and thus a lot of code redundancy. This is to not create any temporary arrays for transcoding to/from different format types. The methods not used can simply be commented out.
  There is also a "fast" version of all decode methods that works the same way as the normal ones, but har a few demands on the decoded input. Normally though, these fast verions should be used if the source if the input is known and it hasn't bee tampered with.
  If you find the code useful or you find a bug, please send me a note at base64 @ miginfocom . com. Licence (BSD): ============== Copyright (c) 2004, Mikael Grev, MiG InfoCom AB. (base64 @ miginfocom . com) All rights reserved. Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. Neither the name of the MiG InfoCom AB nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

- - ### Constructor Summary

    Constructors
    Constructor | Description
    --- | ---
    `Base64()` |

  - ### Method Summary

    All Methods Static Methods Concrete Methods
    Modifier and Type | Method | Description
    --- | --- | ---
    `static byte[]` | `decode​(byte[] sArr)` | Decodes a BASE64 encoded byte array.
    `static byte[]` | `decode​(byte[] sArr, int sOff, int sLen)` | Decodes a BASE64 encoded byte array.
    `static byte[]` | `decode​(char[] sArr)` | Decodes a BASE64 encoded char array.
    `static byte[]` | `decode​(java.lang.String str)` | Decodes a BASE64 encoded `String`.
    `static byte[]` | `decodeFast​(byte[] sArr)` | Decodes a BASE64 encoded byte array that is known to be resonably well formatted.
    `static byte[]` | `decodeFast​(char[] sArr)` | Decodes a BASE64 encoded char array that is known to be resonably well formatted.
    `static byte[]` | `decodeFast​(java.lang.String s)` | Decodes a BASE64 encoded string that is known to be resonably well formatted.
    `static byte[]` | `encodeToByte​(byte[] sArr, boolean lineSep)` | Encodes a raw byte array into a BASE64 `byte[]` representation i accordance with RFC 2045.
    `static byte[]` | `encodeToByte​(byte[] sArr, int sOff, int sLen, boolean lineSep)` | Encodes a raw byte array into a BASE64 `byte[]` representation i accordance with RFC 2045.
    `static char[]` | `encodeToChar​(byte[] sArr, boolean lineSep)` | Encodes a raw byte array into a BASE64 `char[]` representation i accordance with RFC 2045.
    `static java.lang.String` | `encodeToString​(byte[] sArr, boolean lineSep)` | Encodes a raw byte array into a BASE64 `String` representation i accordance with RFC 2045.

    - ### Methods inherited from class java.lang.Object

      `clone, equals, finalize, getClass, hashCode, notify, notifyAll, toString, wait, wait, wait`

- - ### Constructor Detail

    - #### Base64

          public Base64()

  - ### Method Detail

    - #### decode

          public static final byte[] decode​(byte[] sArr)

      Decodes a BASE64 encoded byte array. All illegal characters will be ignored and can handle both arrays with and without line separators.
      Parameters:
        `sArr` - The source array. Length 0 will return an empty array. `null` will throw an exception.
      Returns:
        The decoded array of bytes. May be of length 0. Will be `null` if the legal characters (including '=') isn't divideable by 4. (I.e. definitely corrupted).

    - #### decode

          public static final byte[] decode​(byte[] sArr,
                                            int sOff,
                                            int sLen)

      Decodes a BASE64 encoded byte array. All illegal characters will be ignored and can handle both arrays with and without line separators.
      Parameters:
        `sArr` - The source array. `null` will throw an exception.
        `sOff` - The starting position in the source array.
        `sLen` - The number of bytes to decode from the source array. Length 0 will return an empty array.
      Returns:
        The decoded array of bytes. May be of length 0. Will be `null` if the legal characters (including '=') isn't divideable by 4. (I.e. definitely corrupted).

    - #### decode

          public static final byte[] decode​(char[] sArr)

      Decodes a BASE64 encoded char array. All illegal characters will be ignored and can handle both arrays with and without line separators.
      Parameters:
        `sArr` - The source array. `null` or length 0 will return an empty array.
      Returns:
        The decoded array of bytes. May be of length 0. Will be `null` if the legal characters (including '=') isn't divideable by 4. (I.e. definitely corrupted).

    - #### decode

          public static final byte[] decode​(java.lang.String str)

      Decodes a BASE64 encoded `String`. All illegal characters will be ignored and can handle both strings with and without line separators.
      Note! It can be up to about 2x the speed to call `decode(str.toCharArray())` instead. That will create a temporary array though. This version will use `str.charAt(i)` to iterate the string.
      Parameters:
        `str` - The source string. `null` or length 0 will return an empty array.
      Returns:
        The decoded array of bytes. May be of length 0. Will be `null` if the legal characters (including '=') isn't divideable by 4. (I.e. definitely corrupted).

    - #### decodeFast

          public static final byte[] decodeFast​(byte[] sArr)

      Decodes a BASE64 encoded byte array that is known to be resonably well formatted. The method is about twice as fast as `decode(byte[])`. The preconditions are:
      + The array must have a line length of 76 chars OR no line separators at all (one line).
      + Line separator must be "\r\n", as specified in RFC 2045 + The array must not contain illegal characters within the encoded string
      + The array CAN have illegal characters at the beginning and end, those will be dealt with appropriately.
      Parameters:
        `sArr` - The source array. Length 0 will return an empty array. `null` will throw an exception.
      Returns:
        The decoded array of bytes. May be of length 0.

    - #### decodeFast

          public static final byte[] decodeFast​(char[] sArr)

      Decodes a BASE64 encoded char array that is known to be resonably well formatted. The method is about twice as fast as `decode(char[])`. The preconditions are:
      + The array must have a line length of 76 chars OR no line separators at all (one line).
      + Line separator must be "\r\n", as specified in RFC 2045 + The array must not contain illegal characters within the encoded string
      + The array CAN have illegal characters at the beginning and end, those will be dealt with appropriately.
      Parameters:
        `sArr` - The source array. Length 0 will return an empty array. `null` will throw an exception.
      Returns:
        The decoded array of bytes. May be of length 0.

    - #### decodeFast

          public static final byte[] decodeFast​(java.lang.String s)

      Decodes a BASE64 encoded string that is known to be resonably well formatted. The method is about twice as fast as `decode(String)`. The preconditions are:
      + The array must have a line length of 76 chars OR no line separators at all (one line).
      + Line separator must be "\r\n", as specified in RFC 2045 + The array must not contain illegal characters within the encoded string
      + The array CAN have illegal characters at the beginning and end, those will be dealt with appropriately.
      Parameters:
        `s` - The source string. Length 0 will return an empty array. `null` will throw an exception.
      Returns:
        The decoded array of bytes. May be of length 0.

    - #### encodeToByte

          public static final byte[] encodeToByte​(byte[] sArr,
                                                  boolean lineSep)

      Encodes a raw byte array into a BASE64 `byte[]` representation i accordance with RFC 2045.
      Parameters:
        `sArr` - The bytes to convert. If `null` or length 0 an empty array will be returned.
        `lineSep` - Optional "\r\n" after 76 characters, unless end of file.
        No line separator will be in breach of RFC 2045 which specifies max 76 per line but will be a little faster.
      Returns:
        A BASE64 encoded array. Never `null`.

    - #### encodeToByte

          public static final byte[] encodeToByte​(byte[] sArr,
                                                  int sOff,
                                                  int sLen,
                                                  boolean lineSep)

      Encodes a raw byte array into a BASE64 `byte[]` representation i accordance with RFC 2045.
      Parameters:
        `sArr` - The bytes to convert. If `null` an empty array will be returned.
        `sOff` - The starting position in the bytes to convert.
        `sLen` - The number of bytes to convert. If 0 an empty array will be returned.
        `lineSep` - Optional "\r\n" after 76 characters, unless end of file.
        No line separator will be in breach of RFC 2045 which specifies max 76 per line but will be a little faster.
      Returns:
        A BASE64 encoded array. Never `null`.

    - #### encodeToChar

          public static final char[] encodeToChar​(byte[] sArr,
                                                  boolean lineSep)

      Encodes a raw byte array into a BASE64 `char[]` representation i accordance with RFC 2045.
      Parameters:
        `sArr` - The bytes to convert. If `null` or length 0 an empty array will be returned.
        `lineSep` - Optional "\r\n" after 76 characters, unless end of file.
        No line separator will be in breach of RFC 2045 which specifies max 76 per line but will be a little faster.
      Returns:
        A BASE64 encoded array. Never `null`.

    - #### encodeToString

          public static final java.lang.String encodeToString​(byte[] sArr,
                                                              boolean lineSep)

      Encodes a raw byte array into a BASE64 `String` representation i accordance with RFC 2045.
      Parameters:
        `sArr` - The bytes to convert. If `null` or length 0 an empty array will be returned.
        `lineSep` - Optional "\r\n" after 76 characters, unless end of file.
        No line separator will be in breach of RFC 2045 which specifies max 76 per line but will be a little faster.
      Returns:
        A BASE64 encoded array. Never `null`.